import requests
import socket
from netbox import exceptions


class NetboxConnection(object):

    def __init__(self, ssl_verify=False, use_ssl=True, host=None, auth_token=None, auth=None,
                 port=None, api_prefix=None):
        self.use_ssl = use_ssl
        self.host = host
        self.auth_token = auth_token
        self.port = port
        self.auth = auth
        self.api_prefix = api_prefix

        self.base_url = 'http{s}://{host}{p}{prefix}'.format(s='s' if use_ssl else '', p=':{}'.format(self.port) if self.port else '', host=self.host, prefix='/api' if api_prefix is None else api_prefix)

        self.session = requests.Session()
        self.session.verify = ssl_verify

        if auth:
            self.session.auth = auth

        if auth_token:
            token = 'Token {}'.format(self.auth_token)
            self.session.headers.update({'Authorization': token})
            self.session.headers.update({'Accept': 'application/json'})
            self.session.headers.update({'Content-Type': 'application/json'})

        if auth and auth_token:
            raise ValueError('Only one authentication method is possible. Please use auth or auth_token')

    def __request(self, method, params=None, key=None, body=None, url=None):

        if method is not 'GET':
            if not self.auth_token:
                raise exceptions.AuthException('Authentication credentials were not provided')

            if self.auth:
                raise exceptions.AuthException('With basic authentication the API is not writable.')

        if url is None:
            if key is not None:
                url = self.base_url + str(params) + str('{}/'.format(key))
            else:
                url = self.base_url + str(params)

        request = requests.Request(method=method, url=url, json=body)
        prepared_request = self.session.prepare_request(request)

        try:
            response = self.session.send(prepared_request)
        except socket.gaierror:
            err_msg = 'Unable to find address: {}'.format(self.host)
            raise socket.gaierror(err_msg)
        except requests.exceptions.ConnectionError:
            err_msg = 'Unable to connect to Netbox host: {}'.format(self.host)
            raise ConnectionError(err_msg)
        except requests.exceptions.Timeout:
            raise TimeoutError('Connection to Netbox host timed out')
        except Exception as e:
            raise Exception(e)
        finally:
            self.close()

        try:
            response_data = response.json()
        except:
            response_data = response.content

        return response.ok, response.status_code, response_data

    def get(self, param, key=None, limit=0, **kwargs):

        if kwargs:
            url = '{}{}?{}&limit={}'.format(self.base_url, param,
                                            '&'.join('{}={}'.format(key, val) for key, val in kwargs.items()), limit)
        elif key:
            if '_choices' in param:
                url = '{}{}{}/?limit={}'.format(self.base_url, param, key, limit)
            else:
                url = '{}{}/?q={}&limit={}'.format(self.base_url, param, key, limit)
        else:
            url = '{}{}?limit={}'.format(self.base_url, param, limit)

        resp_ok, resp_status, resp_data = self.__request('GET', params=param, key=key, url=url)

        if resp_ok and resp_status == 200:
            if 'results' in resp_data:
                return resp_data['results']
            else:
                return resp_data
        else:
            return []

    def put(self, params):

        return self.__request('PUT', params)

    def patch(self, params, key, **kwargs):

        body_data = {key: value for (key, value) in kwargs.items()}
        resp_ok, resp_status, resp_data = self.__request('PATCH', params=params, key=key, body=body_data)
        if resp_ok and resp_status == 200:
            return True
        else:
            raise exceptions.UpdateException(resp_data)

    def post(self, params, required_fields, **kwargs):

        body_data = {key: value for (key, value) in required_fields.items()}

        if kwargs:
            body_data.update({key: value for (key, value) in kwargs.items()})

        resp_ok, resp_status, resp_data = self.__request('POST', params=params, body=body_data)
        if resp_ok and resp_status == 201:
            return resp_data
        else:
            raise exceptions.CreateException(resp_data)

    def delete(self, params, del_id):

        del_str = '{}{}'.format(params, del_id)
        resp_ok, resp_status, resp_data = self.__request('DELETE', del_str)
        if resp_ok and resp_status == 204:
            return True
        else:
            raise exceptions.DeleteException(resp_data)

    def close(self):

        self.session.close()


import ipaddress
from netbox import exceptions


class Ipam(object):

    def __init__(self, netbox_con):

        self.netbox_con = netbox_con

    def get_choices(self, choice_id=None):
        """Return choices for all fields if choice_id is not defined

        :param choice_id: Optional model:field tuple
        """
        return self.netbox_con.get('/ipam/_choices/', choice_id)

    def get_services(self, **kwargs):
        """Return all services"""
        return self.netbox_con.get('/ipam/services/', **kwargs)

    def create_service(self, name, port, protocol, **kwargs):
        """Create a new service definition

        :param name: Name of the provided service
        :param port: Service listening port
        :param protocol: Service used protocol 6 is used to TCP and 17 for UDP 
        :param kwargs: Optional arguments (need to attach service to a 'virtual_machine: id' or 'device: id' )
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "port": port, "protocol": protocol }
        return self.netbox_con.post('/ipam/services/', required_fields, **kwargs)

    def get_ip_addresses(self, **kwargs):
        """Return all ip addresses"""
        return self.netbox_con.get('/ipam/ip-addresses/', **kwargs)

    def get_ip_by_device(self, device_name):
        """Get IPs which are associated to a device

        :param device_name: Name of the device
        :return: ip address information
        """
        return self.netbox_con.get('/ipam/ip-addresses', device=device_name)

    def get_ip_by_virtual_machine(self, vm_name):
        """Get IPs which are associated to a virtual-machine

        :param vm_name: Name of the virtual machine
        :return: ip address information
        """
        return self.netbox_con.get('/ipam/ip-addresses', virtual_machine=vm_name)

    def create_ip_address(self, address, **kwargs):
        """Create a new ip address

        :param address: IP address
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"address": address}
        return self.netbox_con.post('/ipam/ip-addresses/', required_fields, **kwargs)

    def update_ip(self, ip_address, **kwargs):
        """Update ip address

        :param ip_address: ip address with prefix. Format: 1.1.1.1/32
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            ip_id = self.get_ip_addresses(address=ip_address)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('ip: {}'.format(ip_address)) from None
        return self.netbox_con.patch('/ipam/ip-addresses/', ip_id, **kwargs)

    def delete_ip_address(self, ip_address):
        """Delete IP address

        :param ip_address: IP address to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            ip_id = self.get_ip_addresses(address=ip_address)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('ip: {}'.format(ip_address)) from None
        return self.netbox_con.delete('/ipam/ip-addresses/', ip_id)

    def delete_ip_by_id(self, ip_id):
        """Delete IP address

        :param ip_id: ID of ip address to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        return self.netbox_con.delete('/ipam/ip-addresses/', ip_id)

    def get_ip_prefixes(self, **kwargs):
        """Return all ip prefixes"""
        return self.netbox_con.get('/ipam/prefixes/', **kwargs)

    def create_ip_prefix(self, prefix, **kwargs):
        """Create a new ip prefix

        :param prefix: A valid ip prefix format. The syntax will be checked with the ipaddress module
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"prefix": prefix}

        if ipaddress.ip_network(prefix, strict=True):
            return self.netbox_con.post('/ipam/prefixes/', required_fields, **kwargs)

    def delete_ip_prefix(self, **kwargs):
        """Delete IP prefix

        :param kwargs: Delete prefix based on filter values
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            ip_prefix_id = self.get_ip_prefixes(**kwargs)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('ip-prefix') from None
        return self.netbox_con.delete('/ipam/prefixes/', ip_prefix_id)

    def update_ip_prefix(self, ip_prefix, **kwargs):
        """Update ip address

        :param ip_prefix: ip prefix to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            ip_prefix_id = self.get_ip_prefixes(prefix=ip_prefix)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('ip-prefix: {}'.format(ip_prefix)) from None
        return self.netbox_con.patch('/ipam/prefixes/', ip_prefix_id, **kwargs)

    def get_next_available_ip(self, **kwargs):
        """Return next available ip in prefix

        :param kwargs: Use the filter fields from the get_prefixes call
        :return: next available ip
        """
        try:
            prefix_id = self.get_ip_prefixes(**kwargs)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('ip-prefix') from None

        param = '/ipam/prefixes/{}/available-ips/'.format(prefix_id)
        return self.netbox_con.get(param, limit=1)[0]['address']

    def get_vrfs(self, **kwargs):
        """Get all vrfs"""
        return self.netbox_con.get('/ipam/vrfs/', **kwargs)

    def create_vrf(self, name, rd, **kwargs):
        """Create a new vrf

        :param name: Name of the vrf
        :param rd: Route distinguisher in any format
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "rd": rd}
        return self.netbox_con.post('/ipam/vrfs/', required_fields, **kwargs)

    def delete_vrf(self, vrf_name):
        """Delete vrf

        :param vrf_name: Name of vrf to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            vrf_id = self.get_vrfs(name=vrf_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('vrf: {}'.format(vrf_name)) from None
        return self.netbox_con.delete('/ipam/vrfs/', vrf_id)

    def update_vrf(self, vrf_name, **kwargs):
        """Update vrf

        :param vrf_name: name of the vrf to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            vrf_id = self.get_vrfs(name=vrf_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('vrf: {}'.format(vrf_name)) from None
        return self.netbox_con.patch('/ipam/vrfs/', vrf_id, **kwargs)

    def get_aggregates(self, **kwargs):
        """Return all aggregates"""
        return self.netbox_con.get('/ipam/aggregates/', **kwargs)

    def create_aggregate(self, prefix, rir, **kwargs):
        """Creates a new aggregate

        :param prefix: IP Prefix
        :param rir: Name of the RIR
        :param kwargs: Optional Arguments
        :return:
        """
        rir_id = self.get_rirs(name=rir)[0]['id']
        required_fields = {"prefix": prefix, "rir": rir_id}

        if ipaddress.ip_network(prefix, strict=True):
            return self.netbox_con.post('/ipam/aggregates/', required_fields, **kwargs)

    def update_aggregate(self, prefix, **kwargs):
        """Update aggregate

        :param prefix: Prefix of the aggregate
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            aggregate_id = self.get_aggregates(prefix=prefix)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('aggregate: {}'.format(prefix)) from None
        return self.netbox_con.patch('/ipam/aggregates/', aggregate_id, **kwargs)

    def get_rirs(self, **kwargs):
        """Return all rirs"""
        return self.netbox_con.get('/ipam/rirs/', **kwargs)

    def create_rir(self, name, slug):
        """Create new rir

        :param name: Name of the rir
        :param slug: Name of the slug
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/ipam/rirs/', required_fields)

    def delete_rir(self, rir_name):
        """Delete rir

        :param rir_name: rir name to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            rir_id = self.get_rirs(name=rir_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rir: {}'.format(rir_name)) from None
        return self.netbox_con.delete('/ipam/rirs/', rir_id)

    def update_rir(self, rir_name, **kwargs):
        """Update rir

        :param rir_name: Name of the rir
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            rir_id = self.get_rirs(name=rir_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rir: {}'.format(rir_name)) from None
        return self.netbox_con.patch('/ipam/rirs/', rir_id, **kwargs)

    def get_roles(self, **kwargs):
        """Return all roles"""
        return self.netbox_con.get('/ipam/roles/', **kwargs)

    def create_role(self, name, slug, **kwargs):
        """Create new prefix/vlan role

        :param name: Name of the prefix/vlan role
        :param slug: Name of the slug
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/ipam/roles/', required_fields, **kwargs)

    def delete_role(self, role_name):
        """Delete prefix/vlan role

        :param role_name: prefix/vlan role to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            role_id = self.get_roles(name=role_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('prefix/vlan role: {}'.format(role_name)) from None
        return self.netbox_con.delete('/ipam/role/', role_id)

    def update_role(self, role_name, **kwargs):
        """Update prefix role

        :param role_name: Name of the prefix/vlan role
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            prefix_role_id = self.get_roles(name=role_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('prefix/vlan role: {}'.format(role_name)) from None
        return self.netbox_con.patch('/ipam/roles/', prefix_role_id, **kwargs)

    def get_vlans(self, **kwargs):
        """Return all vlans"""
        return self.netbox_con.get('/ipam/vlans/', **kwargs)

    def create_vlan(self, vid, vlan_name, **kwargs):
        """Create new vlan

        :param vid: ID of the new vlan
        :param vlan_name: Name of the vlan
        :param kwargs: Optional Arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"vid": vid, "name": vlan_name}
        return self.netbox_con.post('/ipam/vlans/', required_fields, **kwargs)

    def delete_vlan(self, vid):
        """Delete VLAN based on VLAN ID

        :param vid: vlan id to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            vid_id = self.get_vlans(vid=vid)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('vlan: {}'.format(vid)) from None
        return self.netbox_con.delete('/ipam/vlans/', vid_id)

    def update_vlan(self, vlan_name, **kwargs):
        """Update vlan

        :param vlan_name: Name of the vlan
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            vlan_id = self.get_vlans(name=vlan_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('vlan: {}'.format(vlan_name)) from None
        return self.netbox_con.patch('/ipam/vlans/', vlan_id, **kwargs)

    def get_vlan_groups(self, **kwargs):
        """Return all vlan groups"""
        return self.netbox_con.get('/ipam/vlan-groups/', **kwargs)

    def create_vlan_group(self, name, slug, **kwargs):
        """Create new vlan-group

        :param name: name of the vlan group
        :param slug: slug
        :param kwargs: Optional Arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/ipam/vlan-groups/', required_fields, **kwargs)

    def delete_vlan_group(self, name):
        """Delete VLAN group

        :param name: name of the vlan-group to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            vgrp_id = self.get_vlan_groups(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('vlan: {}'.format(name)) from None
        return self.netbox_con.delete('/ipam/vlan-groups/', vgrp_id)

    def update_vlan_group(self, name, **kwargs):
        """Update vlan-group

        :param name: name of the vlan-group
        :param kwargs: arguments
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            vgrp_ip = self.get_vlan_groups(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('name: {}'.format(name)) from None
        return self.netbox_con.patch('/ipam/vlan-groups/', vgrp_ip, **kwargs)



import netbox.exceptions as exceptions


class Tenancy(object):

    def __init__(self, netbox_con):
        self.netbox_con = netbox_con

    def get_choices(self, choice_id=None):
        """Return choices for all fields if choice_id is not defined

        :param choice_id: Optional model:field tuple
        """
        return self.netbox_con.get('/tenancy/_choices/', choice_id)

    def get_tenants(self, **kwargs):
        """Returns the tenants"""
        return self.netbox_con.get('/tenancy/tenants/', **kwargs)

    def create_tenant(self, name, slug, **kwargs):
        """Create a new tenant

        :param name: Tenant name
        :param slug: slug name
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/tenancy/tenants/', required_fields, **kwargs)

    def delete_tenant(self, tenant_name):
        """Delete tenant

        :param tenant_name: Tenant to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            tenant_id = self.get_tenants(name=tenant_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tenant: {}'.format(tenant_name)) from None
        return self.netbox_con.delete('/tenancy/tenants/', tenant_id)

    def update_tenant(self, tenant_name, **kwargs):
        """Update tenant

        :param tenant_name: tenant to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            tenant_id = self.get_tenants(name=tenant_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tenant: {}'.format(tenant_name)) from None
        return self.netbox_con.patch('/tenancy/tenants/', tenant_id, **kwargs)

    def get_tenant_groups(self, **kwargs):
        """Returns the tenant groups"""
        return self.netbox_con.get('/tenancy/tenant-groups/', **kwargs)

    def create_tenant_group(self, name, slug, **kwargs):
        """Create a new tenant-group

        :param name: Tenant-group name
        :param slug: slug name
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/tenancy/tenant-groups/', required_fields, **kwargs)

    def delete_tenant_group(self, tenant_group_name):
        """Delete tenant

        :param tenant_group_name: Tenant group to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            tenant_group_id = self.get_tenant_groups(name=tenant_group_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tenant: {}'.format(tenant_group_name)) from None
        return self.netbox_con.delete('/tenancy/tenant-groups/', tenant_group_id)

    def update_tenant_group(self, tenant_group_name, **kwargs):
        """Update tenant group

        :param tenant_group_name: tenant group to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            tenant_group_id = self.get_tenant_groups(name=tenant_group_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tenant-group: {}'.format(tenant_group_name)) from None
        return self.netbox_con.patch('/tenancy/tenant-groups/', tenant_group_id, **kwargs)



import netbox.exceptions as exceptions


class Virtualization(object):

    def __init__(self, netbox_con):
        self.netbox_con = netbox_con

    def get_choices(self, choice_id=None):
        """Return choices for all fields if choice_id is not defined

        :param choice_id: Optional model:field tuple
        """
        return self.netbox_con.get('/virtualization/_choices/', choice_id)

    def get_clusters(self, **kwargs):
        """Return all clusters"""
        return self.netbox_con.get('/virtualization/clusters/', **kwargs)

    def create_cluster(self, name, type, **kwargs):
        """Create a new cluster

        :param name: name of the cluster
        :param type: cluster type
        :param kwargs: optional arguments
        :return: netbox object if successful otherwise exception raised
        """
        try:
            cluster_type_id = self.get_cluster_types(name=type)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('cluster-type: {}'.format(name)) from None

        required_fields = {"name": name, "type": cluster_type_id}
        return self.netbox_con.post('/virtualization/clusters/', required_fields, **kwargs)

    def delete_cluster(self, name):
        """Delete a cluster

        :param name: name of the cluster to delete
        :return: netbox object if succesful otherwase delete exception
        """
        try:
            cluster_id = self.get_clusters(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('cluster {}'.format(name)) from None
        return self.netbox_con.delete('/virtualization/clusters/', cluster_id)

    def update_cluster(self, name, **kwargs):
        """Update cluster

        :param kwargs: filter arguments
        :param name: cluster name to update
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            cluster_id = self.get_clusters(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('cluster: {}'.format(name)) from None
        return self.netbox_con.patch('/virtualization/clusters/', cluster_id, **kwargs)

    def get_cluster_types(self, **kwargs):
        """Return all cluster types"""
        return self.netbox_con.get('/virtualization/cluster-types/', **kwargs)

    def create_cluster_type(self, name, slug):
        """Create a new cluster type

        :param name: name of the cluster
        :param slug: slug name
        :return:  netbox object if successful otherwise create exception
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/virtualization/cluster-types/', required_fields)

    def update_cluster_type(self, name, **kwargs):
        """Update cluster type

        :param name: name of the cluster type
        :param kwargs: fields to update
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            type_id = self.get_cluster_types(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException(name) from None
        return self.netbox_con.patch('/virtualization/cluster-types/', type_id, **kwargs)

    def delete_cluster_type(self, name):
        """Delete a cluster type

        :param name: name of the cluster type to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            cluster_type_id = self.get_cluster_types(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('cluster-type: {}'.format(name)) from None
        return self.netbox_con.delete('/virtualization/cluster-types/', cluster_type_id)

    def get_interfaces(self, **kwargs):
        """Return all interfaces"""
        return self.netbox_con.get('/virtualization/interfaces/', **kwargs)

    def get_interface(self, **kwargs):
        """Return interface by filter"""
        return self.netbox_con.get('/virtualization/interfaces/', **kwargs)

    def create_interface(self, name, virtual_machine, **kwargs):
        """Create an interface for a virtual machine

        :param name: name of the interface
        :param virtual_machine: name of virtual machine to attach interface
        :return: netbox object if successful otherwise raise CreateException
        """
        try:
            virtual_machine_id = self.get_virtual_machine(name=virtual_machine)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('virtual-machine: {}'.format(virtual_machine)) from None
        required_fields = {"name": name, "virtual_machine": virtual_machine_id}
        return self.netbox_con.post('/virtualization/interfaces/', required_fields, **kwargs)

    def update_interface(self, name, virtual_machine, **kwargs):
        """Update virtual_machine interface

        :param name: name of the interface to update
        :param virtual_machine: name of the virtual-machine to update
        :param kwargs: update data
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            interface_id = self.get_interface(name=name, virtual_machine=virtual_machine)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface: {} or virtual_machine: {}'
                                               .format(name, virtual_machine)) from None
        return self.netbox_con.patch('/virtualization/interfaces/', interface_id, **kwargs)

    def delete_interface(self, name, virtual_machine):
        """Delete interface from virtual_machine

        :param name: interface name to delete
        :param virtual_machine: machine to delete interface from
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            interface_id = self.get_interface(name=name, virtual_machine=virtual_machine)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface: {} or virtual_machine {}'
                                               .format(name, virtual_machine)) from None

        return self.netbox_con.delete('/virtualization/interfaces/', interface_id)

    def get_virtual_machines(self, **kwargs):
        """Return all virtual-machines"""
        return self.netbox_con.get('/virtualization/virtual-machines/', **kwargs)

    def get_virtual_machine(self, **kwargs):
        """Return virtual-machine based on filter"""
        return self.netbox_con.get('/virtualization/virtual-machines/', **kwargs)

    def create_virtual_machine(self, name, cluster_name, **kwargs):
        """Create a virtual machine
    
        :param name: name of the virtual machine
        :param cluster_name: Name of existing cluster
        :return: netbox object if successful otherwise raise CreateException
        """
        try:
            cluster_id = self.get_clusters(name=cluster_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('cluster-name: {}'.format(cluster_name)) from None

        required_fields = {"name": name, "cluster": cluster_id}
        return self.netbox_con.post('/virtualization/virtual-machines/', required_fields, **kwargs)

    def delete_virtual_machine(self, virtual_machine_name):
        """Delete virtual machine

        :param virtual_machine_name: name of the virtual machine to delete
        :return: bool True if successful otherwise raise exception
        """
        try:
            virtual_machine_id = self.get_virtual_machines(name=virtual_machine_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('virtual-machine: {}'.format(virtual_machine_name)) from None
        return self.netbox_con.delete('/virtualization/virtual-machines/', virtual_machine_id)

    def update_virtual_machine(self, virtual_machine_name, **kwargs):
        """Update virtual-machine

        :param virtual_machine_name: name of the virtual-machine to update
        :param kwargs: update data
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            virtual_machine_id = self.get_virtual_machines(name=virtual_machine_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('virtual-machine: {}'
                                               .format(virtual_machine_name)) from None
        return self.netbox_con.patch('/virtualization/virtual-machines/', virtual_machine_id, **kwargs)



import netbox.exceptions as exceptions


class Extras(object):

    def __init__(self, netbox_con):
        self.netbox_con = netbox_con

    def get_config_contexts(self, **kwargs):
        """Returns all config-contexts"""
        return self.netbox_con.get('/extras/config-contexts/', **kwargs)

    def create_config_context(self, name, data, **kwargs):
        """Create a config-context

        :param name: config-context name
        :param data: data in json format
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "data": data}
        return self.netbox_con.post('/extras/config-contexts/', required_fields, **kwargs)

    def delete_config_context(self, name):
        """Delete config-context

        :param name: Name of the config-context to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            config_context_id = self.get_config_contexts(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('config-context: {}'.format(name)) from None
        return self.netbox_con.delete('/extras/config-contexts/', config_context_id)

    def update_config_context(self, name, **kwargs):
        """Update config-context

        :param name: config-context to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            config_context_id = self.get_config_contexts(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('config-context: {}'.format(name)) from None

        return self.netbox_con.patch('/extras/config-contexts/', config_context_id, **kwargs)

    def get_tags(self, **kwargs):
        """Returns all tags"""
        return self.netbox_con.get('/extras/tags/', **kwargs)

    def create_tag(self, name, slug, **kwargs):
        """Create a tag

        :param name: tag name
        :param slug: tag slug
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/extras/tags/', required_fields, **kwargs)

    def delete_tag(self, name):
        """Delete tag

        :param name: Name of the tag to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            tag_id = self.get_tags(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tag: {}'.format(name)) from None
        return self.netbox_con.delete('/extras/tags/', tag_id)

    def update_tag(self, name, **kwargs):
        """Update tag

        :param name: tag name
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            tag_id = self.get_tags(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('tag: {}'.format(name)) from None

        return self.netbox_con.patch('/extras/tags/', tag_id, **kwargs)

    def get_object_changes(self, **kwargs):
        """Returns all object changes"""
        return self.netbox_con.get('/extras/object-changes/', **kwargs)

    def get_reports(self):
        """Returns all reports"""
        return self.netbox_con.get('/extras/reports/')



class BaseExceptions(Exception):
    pass


class DeleteException(BaseException):
    """Raised when delete failed"""
    def __init__(self, resp_data):

        if isinstance(resp_data, dict):
            self.err = ''.join('Failed with reason {}'.format(val) for key, val in resp_data.items())
        else:
            self.err = 'Delete failed with an unknown reason'

    def __str__(self):
        return '{}'.format(self.err)


class NotFoundException(BaseException):
    """Raised when item is not found"""
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return 'Unable to found {}'.format(self.msg)


class CreateException(BaseException):
    """Raised when creation failed"""
    def __init__(self, resp_data):

        if isinstance(resp_data, dict):
            self.err = ''.join('{} '.format(val[0]) for key, val in resp_data.items())
        else:
            self.err = 'Creation failed with unknown reason'

    def __str__(self):
        return '{}'.format(self.err)


class UpdateException(BaseException):
    """Raised when an object update fails"""
    def __init__(self, resp_data):
        if isinstance(resp_data, dict):
            self.err = ''.join('{} '.format(val[0]) for key, val in resp_data.items())
        else:
            self.err = 'Update failed with unknown reason'

    def __str__(self):
        return '{}'.format(self.err)


class AuthException(BaseException):
    """Raised when an API call method is not allowed"""
    pass



import netbox.connection as connection
import netbox.dcim as dcim
import netbox.ipam as ipam
import netbox.virtualization as virtualization
import netbox.exceptions as exceptions
import netbox.tenancy as tenancy
import netbox.extras as extras


class NetBox(object):
    def __init__(self, host, **kwargs):
        self.connection = connection.NetboxConnection(host=host, **kwargs)
        self.ipam = ipam.Ipam(self.connection)
        self.dcim = dcim.Dcim(self.connection)
        self.virtualization = virtualization.Virtualization(self.connection)
        self.tenancy = tenancy.Tenancy(self.connection)
        self.extras = extras.Extras(self.connection)
        self.exceptions = exceptions



#!/usr/bin/env python

import os
from setuptools import setup, find_packages


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


def readme():
    with open('README.rst') as f:
        return f.read()


setup(name='python-netbox',
      version='0.0.15',
      description='Python NetBox Client',
      long_description=readme(),
      python_requires='>=3',
      author='Thomas van der Jagt',
      author_email='thomas@tjrb.nl',
      url='https://github.com/jagter/python-netbox',
      download_url='https://github.com/jagter/python-netbox/releases/tag/0.0.15.tar.gz',
      packages=find_packages(),
      install_requires=['ipaddress', 'requests'],
      classifiers = [
        "Programming Language :: Python :: 3",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.6",
      ],
     )



import netbox.exceptions as exceptions


class Dcim(object):

    def __init__(self, netbox_con):
        self.netbox_con = netbox_con

    def get_choices(self, choice_id=None):
        """Return choices for all fields if choice_id is not defined

        :param choice_id: Optional model:field tuple
        """
        return self.netbox_con.get('/dcim/_choices/', choice_id)

    def get_regions(self, **kwargs):
        """Returns the available regions"""
        return self.netbox_con.get('/dcim/regions/', **kwargs)

    def create_region(self, name, slug, **kwargs):
        """Create a new region

        :param name: Region name
        :param slug: slug name
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/dcim/regions/', required_fields, **kwargs)

    def delete_region(self, region_name):
        """Delete region

        :param region_name: Region to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            region_id = self.get_regions(name=region_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('region: {}'.format(region_name)) from None
        return self.netbox_con.delete('/dcim/regions/', region_id)

    def update_region(self, region_name, **kwargs):
        """

        :param region_name: Region to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            region_id = self.get_regions(name=region_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('region: {}'.format(region_name)) from None
        return self.netbox_con.patch('/dcim/regions/', region_id, **kwargs)

    def get_sites(self, **kwargs):
        """Returns all available sites"""
        return self.netbox_con.get('/dcim/sites/', **kwargs)

    def create_site(self, name, slug, **kwargs):
        """Create a new site

        :param name: Site name
        :param slug: slug name
        :param kwargs: optional fields
        :return: netbox object if successful otherwise exception raised
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/dcim/sites/', required_fields, **kwargs)

    def delete_site(self, site_name):
        """Delete site

        :param site_name: Site to delete
        :return: bool True if succesful otherwase delete exception
        """
        try:
            site_id = self.get_sites(name=site_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('site: {}'.format(site_name)) from None
        return self.netbox_con.delete('/dcim/sites/', site_id)

    def update_site(self, site_name, **kwargs):
        """

        :param site_name: Site to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            site_id = self.get_sites(name=site_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('site: {}'.format(site_name)) from None
        return self.netbox_con.patch('/dcim/sites/', site_id, **kwargs)

    def get_racks(self, **kwargs):
        """Returns all available racks"""
        return self.netbox_con.get('/dcim/racks/', **kwargs)

    def create_rack(self, name, site_name, **kwargs):
        """Create new rack

        :param name: Organizational rack name
        :param site_name: The site at which the rack exists
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise create exception
        """
        try:
            site_id = self.get_sites(name=site_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('site: {}'.format(site_name)) from None
        required_fields = {"name": name, "site": site_id}
        return self.netbox_con.post('/dcim/racks/', required_fields, **kwargs)

    def delete_rack(self, rack_name):
        """Delete rack

        :param rack_name: Name of the rack to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            rack_id = self.get_racks(name=rack_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rack: {}'.format(rack_name)) from None
        return self.netbox_con.delete('/dcim/racks/', rack_id)

    def update_rack(self, rack_name, **kwargs):
        """

        :param rack_name: rack_name name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            rack_id = self.get_racks(facility_id=rack_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rack: {}'.format(rack_name)) from None
        return self.netbox_con.patch('/dcim/racks/', rack_id, **kwargs)

    def get_rack_groups(self, **kwargs):
        """Returns all available rack groups"""
        return self.netbox_con.get('/dcim/rack-groups/', **kwargs)

    def create_rack_group(self, name, slug, site_name, **kwargs):
        """Create new rack group

        :param name: Rack group name
        :param slug: slug name
        :param site_name: The site at which the rack exists
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise create exception
        """
        try:
            site_id = self.get_sites(name=site_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('site: {}'.format(site_name)) from None
        required_fields = {"name": name, "slug": slug, "site": site_id}
        return self.netbox_con.post('/dcim/rack-groups/', required_fields, **kwargs)

    def delete_rack_group(self, name):
        """Delete rack group

        :param name: Name of the rack group to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            rack_group_id = self.get_rack_groups(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rack-group: {}'.format(name)) from None
        return self.netbox_con.delete('/dcim/rack-groups/', rack_group_id)

    def update_rack_group(self, name, **kwargs):
        """

        :param name: Rack group name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            rack_group_id = self.get_rack_groups(name=name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rack group: {}'.format(name)) from None
        return self.netbox_con.patch('/dcim/rack-groups/', rack_group_id, **kwargs)

    def get_devices(self, **kwargs):
        """Get all devices"""
        return self.netbox_con.get('/dcim/devices/', **kwargs)

    def get_devices_per_rack(self, rack_name, **kwargs):
        """Get devices which belongs to the given rack

        :param rack_name: Name of the rack
        :return: list of devices otherwise an empty list
        """
        try:
            rack_id = self.get_racks(name=rack_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('rack: {}'.format(rack_name)) from None
        return self.netbox_con.get('/dcim/devices', rack_id=rack_id, **kwargs)

    def create_device(self, name, device_role, site_name, device_type, **kwargs):
        """Create a new device

        :param name: Name of the device
        :param device_role: Device role for the device
        :param site_name: Name of the site where the device is created
        :param device_type: Type for the new device
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name}
        try:
            device_role_id = self.get_device_roles(name=device_role)[0]['id']
            required_fields.update({"device_role": device_role_id})
        except IndexError:
            raise exceptions.NotFoundException('device-role: {}'.format(device_role)) from None

        try:
            site_id = self.get_sites(name=site_name)[0]['id']
            required_fields.update({"site": site_id})
        except IndexError:
            raise exceptions.NotFoundException('site: {}'.format(site_name)) from None

        try:
            device_type_id = self.get_device_types(model=device_type)[0]['id']
            required_fields.update({"device_type": device_type_id})
        except IndexError:
            raise exceptions.NotFoundException('device-type: {}'.format(device_type)) from None

        return self.netbox_con.post('/dcim/devices/', required_fields, **kwargs)

    def delete_device(self, device_name):
        """Delete device by name

        :param device_name: Device to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            device_id = self.get_devices(name=device_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device: {}'.format(device_name)) from None
        return self.netbox_con.delete('/dcim/devices/', device_id)

    def update_device(self, device_name, **kwargs):
        """Update device by device name

        :param device_name: device name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        device_id = self.get_devices(name=device_name)[0]['id']
        return self.netbox_con.patch('/dcim/devices/', device_id, **kwargs)

    def get_device_types(self, **kwargs):
        """Get devices by device type"""
        return self.netbox_con.get('/dcim/device-types/', **kwargs)

    def create_device_type(self, model, slug, manufacturer, **kwargs):
        """Create device type

        :param model: Model name
        :param slug: Slug name
        :param manufacturer: Name of the manufacurer
        :param kwargs: optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"model": model, "slug": slug, "manufacturer": manufacturer}
        return self.netbox_con.post('/dcim/device-types/', required_fields, **kwargs)

    def update_device_type(self, device_type, **kwargs):
        """Update device type

        :param device_type: device-type name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            device_type_id = self.get_device_types(model=device_type)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device-type: {}'.format(device_type)) from None
        return self.netbox_con.patch('/dcim/device-types/', device_type_id, **kwargs)

    def delete_device_type(self, model_name):
        """Delete device type

        :param model_name: Name of the model
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            device_type_id = self.get_device_types(model=model_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device-type: {}'.format(model_name)) from None
        return self.netbox_con.delete('/dcim/device-types/', device_type_id)

    def get_device_roles(self, **kwargs):
        """Return all the device roles"""
        return self.netbox_con.get('/dcim/device-roles/', **kwargs)

    def create_device_role(self, name, color, slug, **kwargs):
        """Create device role

        :param name: Role name
        :param color: HTML color code
        :param slug: Slug name
        :param kwargs: optional arguments
        :return: netbox object if successful otherwise CreateException
        """
        required_fields = {"name": name, "color": color, "slug": slug}
        return self.netbox_con.post('/dcim/device-roles/', required_fields, **kwargs)

    def update_device_role(self, device_role, **kwargs):
        """Update device role

        :param device_role: device-type name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            device_role_id = self.get_device_roles(name=device_role)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device-role: {}'.format(device_role)) from None
        return self.netbox_con.patch('/dcim/device-roles/', device_role_id, **kwargs)

    def delete_device_role(self, device_role):
        """Delete device by device role

        :param device_role: name of the role
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            device_role_id = self.get_device_roles(name=device_role)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device-role: {}'.format(device_role)) from None
        return self.netbox_con.delete('/dcim/device-roles/', device_role_id)

    def get_manufacturers(self, **kwargs):
        """Return all manufactures"""
        return self.netbox_con.get('/dcim/manufacturers/', **kwargs)

    def create_manufacturer(self, name, slug, **kwargs):
        """Create new manufacturer

        :param name: Name of manufacturer
        :param slug: Name of slug
        :param kwargs: Optional arguments
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/dcim/manufacturers/', required_fields, **kwargs)

    def update_manufacturer(self, manufacturer_name, **kwargs):
        """Update manufacturer

        :param manufacturer_name: manufacturer name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            manufacturer_id = self.get_manufacturers(name=manufacturer_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('manufacturer: {}'.format(manufacturer_name)) from None
        return self.netbox_con.patch('/dcim/manufacturer/', manufacturer_id, **kwargs)

    def delete_manufacturer(self, manufacturer_name):
        """Delete manufacturer

        :param manufacturer_name: Name of manufacturer to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            manufacturer_id = self.get_manufacturers(name=manufacturer_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('manufacturer: {}'.format(manufacturer_name)) from None
        return self.netbox_con.delete('/dcim/manufacturers/', manufacturer_id)

    def get_platforms(self, **kwargs):
        """Return all platforms"""
        return self.netbox_con.get('/dcim/platforms', **kwargs)

    def create_platform(self, name, slug, **kwargs):
        """Create new platform

        :param name: Name of platform
        :param slug: Name of slug
        :param kwargs: Optional arguments
        :return:
        """
        required_fields = {"name": name, "slug": slug}
        return self.netbox_con.post('/dcim/platforms/', required_fields, **kwargs)

    def update_platform(self, platform_name, **kwargs):
        """Update platform

        :param platform_name: device name to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            platform_id = self.get_platforms(name=platform_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('platform: {}'.format(platform_name)) from None
        return self.netbox_con.patch('/dcim/platforms/', platform_id, **kwargs)

    def delete_platform(self, platform_name):
        """Delete platform

        :param platform_name: Name of platform to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            platform_id = self.get_platforms(name=platform_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('platform: {}'.format(platform_name)) from None
        return self.netbox_con.delete('/dcim/platforms/', platform_id)

    def get_interfaces(self, **kwargs):
        """Return interfaces"""
        return self.netbox_con.get('/dcim/interfaces', **kwargs)

    def create_interface(self, name, interface_type, device_id, **kwargs):
        """Create a new interface

        :param name: name of the interface
        :param interface_type: interface type. It is not possible to get the list of types from the api. Search in the netbox code for the correct type number.
        :param kwargs: optional arguments
        :param device_id: ID of the device to associate interface with
        :return: netbox object if successful otherwise raise CreateException
        """
        required_fields = {"name": name, "type": interface_type, "device": device_id}
        return self.netbox_con.post('/dcim/interfaces/', required_fields, **kwargs)

    def update_interface(self, interface, device, **kwargs):
        """Update interface

        :param interface: interface to update
        :param device: name of the device
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            interface_id = self.get_interfaces(name=interface, device=device)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface: {}'.format(interface)) from None
        return self.netbox_con.patch('/dcim/interfaces/', interface_id, **kwargs)

    def delete_interface(self, interface_name, device):
        """Delete interface

        :param interface_name: Name of interface to delete
        :param device: Device to which the interface belongs
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            interface_id = self.get_interfaces(name=interface_name, device=device)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface: {}'.format(interface_name)) from None
        return self.netbox_con.delete('/dcim/interfaces/', interface_id)

    def get_interface_connections(self, **kwargs):
        """Get interface connections

        :param kwargs: Filter arguments
        :return: list of interface connections
        """
        return self.netbox_con.get('/dcim/interface-connections/', **kwargs)

    def create_interface_connection(self, interface_a, interface_b, **kwargs):
        """Create a new interface-connection

        :param interface_a: id of the source interface
        :param interface_b: id of the destination interface
        :param kwargs:
        :return:
        """
        required_fields = {"interface_a": interface_a, "interface_b": interface_b}
        return self.netbox_con.post('/dcim/interface-connections/', required_fields, **kwargs)

    def delete_interface_connection(self, interface_connection_id):
        """Delete interface-connection by id

        :param interface_connection_id: id of interface-connection to remove
        :return: bool True if successful otherwise raise DeleteException
        """
        return self.netbox_con.delete('/dcim/interface-connections/', interface_connection_id)

    def update_interface_connection(self, interface_connection_id, **kwargs):
        """Update interface

        :param interface_connection_id: interface_connection to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        return self.netbox_con.patch('/dcim/interface-connections/', interface_connection_id, **kwargs)

    def get_interface_templates(self, **kwargs):
        """Return interface templates"""
        return self.netbox_con.get('/dcim/interface-templates', **kwargs)

    def create_interface_template(self, name, device_type, **kwargs):
        """Create a new interface template

        :param name: name of the interface
        :param kwargs: optional arguments
        :param device_type: name of the device_type to associate template with
        :return: netbox object if successful otherwise raise CreateException
        """
        try:
            device_type_id = self.get_device_types(model=device_type)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device-type: {}'.format(device_type)) from None
        required_fields = {"name": name, "device_type": device_type_id}
        return self.netbox_con.post('/dcim/interface-templates/', required_fields, **kwargs)

    def update_interface_template(self, interface_template_name, **kwargs):
        """Update interface template

        :param interface_template_name: interface template to update
        :param kwargs: requests body dict
        :return: bool True if successful otherwise raise UpdateException
        """
        try:
            interface_template_id = self.get_interface_templates(name=interface_template_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface: {}'.format(interface_template_name)) from None
        return self.netbox_con.patch('/dcim/interface-templates/', interface_template_id, **kwargs)

    def delete_interface_template(self, interface_template_name):
        """Delete interface template

        :param interface_template_name: Name of interface template to delete
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            interface_template_id = self.get_interface_templates(name=interface_template_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('interface-template: {}'.format(interface_template_name)) from None
        return self.netbox_con.delete('/dcim/interface-templates/', interface_template_id)

    def get_inventory_items(self, **kwargs):
        """Return inventory items"""
        return self.netbox_con.get('/dcim/inventory-items/', **kwargs)

    def create_inventory_item(self, name, device_name, **kwargs):
        """Create inventory item

        :param name: Inventory item name
        :param device_name: Name of device
        :param kwargs: Extra inventory parameters
        :return: netbox object if successful otherwise raise CreateException
        """
        try:
            device_id = self.get_devices(name=device_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('device: {}'.format(device_name)) from None
        required_fields = {"name": name, "device": device_id}
        return self.netbox_con.post('/dcim/inventory-items/', required_fields, **kwargs)

    def update_inventory_item(self, name, device_name, **kwargs):
        """Update inventory item

        :param name: Inventory item name
        :param device_name: Name of device
        :param kwargs: Extra inventory items to update
        :return bool True if successful otherwise raise UpdateException
        """
        try:
            inventory_item_id = self.get_inventory_items(name=name, device=device_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('inventory item: {}'.format(name)) from None
        return self.netbox_con.patch('/dcim/inventory-items/', inventory_item_id, **kwargs)

    def delete_inventory_item(self, name, device_name):
        """Delete inventory item

        :param name: Name of inventory item to delete
        :param device_name: Name of the device to remove inventory item from
        :return: bool True if successful otherwise raise DeleteException
        """
        try:
            inventory_item_id = self.get_inventory_items(name=name, device=device_name)[0]['id']
        except IndexError:
            raise exceptions.NotFoundException('inventory item: {}'.format(name)) from None
        return self.netbox_con.delete('/dcim/inventory-items/', inventory_item_id)
